"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GenericParser = void 0;
var saxophone_ts_1 = require("saxophone-ts");
var Tag = /** @class */ (function () {
    function Tag(tag) {
        if (tag) {
            this.update(tag);
        }
    }
    Tag.prototype.update = function (tag) {
        this.name = tag.name.split(':').reverse()[0];
        if (tag.attrs)
            this.attrs = saxophone_ts_1.parseAttrs(tag.attrs);
    };
    Tag.prototype.toObject = function () {
        var resp = {};
        if (this.text) {
            return this.text;
        }
        if (this.next) {
            for (var _i = 0, _a = this.next; _i < _a.length; _i++) {
                var next = _a[_i];
                // if already exists
                if (resp[next.name]) {
                    if (!(resp[next.name] instanceof Array)) {
                        resp[next.name] = [resp[next.name]];
                    }
                    resp[next.name].push(next.toObject());
                }
                else {
                    resp[next.name] = next.toObject();
                }
            }
        }
        return resp;
    };
    return Tag;
}());
var GenericParser = /** @class */ (function () {
    function GenericParser() {
        this.parser = new saxophone_ts_1.Saxophone();
        this.parser.on('tagOpen', this.onTagOpen.bind(this));
        this.parser.on('tagClose', this.onTagClose.bind(this));
        this.parser.on('text', this.onTextOrCdata.bind(this));
        this.parser.on('cdata', this.onTextOrCdata.bind(this));
    }
    GenericParser.prototype.onTagOpen = function (tag) {
        var newTag = new Tag(tag);
        if (this.currentTag) {
            if (!this.currentTag.next) {
                this.currentTag.next = [];
            }
            this.currentTag.next.push(newTag);
            newTag.previous = this.currentTag;
        }
        this.currentTag = newTag;
    };
    GenericParser.prototype.onTagClose = function (tag) {
        if (this.currentTag.previous) {
            this.currentTag = this.currentTag.previous;
        }
    };
    GenericParser.prototype.onTextOrCdata = function (text) {
        var trimmed = text.contents.trim();
        if (trimmed.length > 0 && trimmed.indexOf('\n') === -1) {
            this.currentTag.text = text.contents.trim();
        }
    };
    GenericParser.prototype.parse = function (xml) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.parser.on('finish', function () {
                resolve(_this.currentTag.toObject());
            });
            _this.parser.on('error', function (err) {
                reject(err);
            });
            _this.parser.parse(xml);
        });
    };
    return GenericParser;
}());
exports.GenericParser = GenericParser;
