"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SubscriptionParser = void 0;
var saxophone_ts_1 = require("saxophone-ts");
var SubscriptionParser = /** @class */ (function () {
    function SubscriptionParser() {
        this.response = {};
        this.parser = new saxophone_ts_1.Saxophone();
        this.parser.on('tagOpen', this.onTagOpen.bind(this));
        this.parser.on('tagClose', this.onTagClose.bind(this));
        this.parser.on('text', this.onTextOrCdata.bind(this));
        this.parser.on('cdata', this.onTextOrCdata.bind(this));
    }
    SubscriptionParser.prototype.cleanTagName = function (tagName) {
        return tagName.split(':').reverse()[0];
    };
    SubscriptionParser.prototype.onTagOpen = function (tag) {
        if (tag.isSelfClosing) {
            return;
        }
        var tagName = this.cleanTagName(tag.name);
        if (this.currentXPath) {
            this.currentXPath = this.currentXPath + '.' + tagName;
        }
        else {
            this.currentXPath = tagName;
        }
        for (var _i = 0, _a = this.request; _i < _a.length; _i++) {
            var sub = _a[_i];
            if (this.currentXPath === sub.tag) {
                this.found = sub;
                return;
            }
        }
    };
    SubscriptionParser.prototype.onTagClose = function (tag) {
        this.found = null;
        this.currentXPath = this.currentXPath.substr(0, this.currentXPath.lastIndexOf('.'));
    };
    SubscriptionParser.prototype.onTextOrCdata = function (text) {
        if (this.found && text.contents && text.contents.trim().length > 0) {
            var key = this.found.name;
            var value = this.found.converter ? this.found.converter.convert(text.contents.trim()) : text.contents.trim();
            // if element already in the response convert to array
            if (key in this.response) {
                if (!(this.response[key] instanceof Array)) {
                    this.response[key] = [this.response[key]];
                }
                this.response[key].push(value);
            }
            else {
                this.response[key] = value;
            }
        }
    };
    SubscriptionParser.prototype.subscribe = function (request) {
        this.request = request;
        return this;
    };
    SubscriptionParser.prototype.parse = function (xml) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.parser.on('finish', function () {
                resolve(_this.response);
            });
            _this.parser.on('error', function (err) {
                reject(err);
            });
            _this.parser.parse(xml);
        });
    };
    return SubscriptionParser;
}());
exports.SubscriptionParser = SubscriptionParser;
